@@ -744,11 +744,16 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     )
 
     if (feature == "exon") {
-        exon_rank <- .extract_rank_from_id(exon_id, tx_id)
-        if (is.null(exon_rank))
-            exon_rank <- .infer_rank_from_position(tx_id,
-                                                   exon_chrom, exon_strand,
-                                                   exon_start, exon_end)
+        exon_rank <- .infer_rank_from_position(tx_id,
+                                               exon_chrom, exon_strand,
+                                               exon_start, exon_end)
+        na_idx <- which(is.na(exon_rank))
+        if (length(na_idx) != 0L) {
+            ## Try to infer the exon ranks from their ids.
+            exon_rank2 <- .extract_rank_from_id(exon_id, tx_id)
+            if (!is.null(exon_rank2))
+                exon_rank[na_idx] <- exon_rank2[na_idx]
+        }
         exons$exon_rank <- exon_rank
     } else {
         colnames(exons) <- sub("^exon_", "", colnames(exons))
@@ -1006,7 +1011,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
 {
     cds_tx_id <- factor(cds$tx_id, levels=levels(exons$tx_id))
     if (any(is.na(cds_tx_id)))
-        stop(wmsg("some CDS cannot be mapped to an exon"))
+        stop(wmsg("some CDS parts cannot be mapped to an exon"))
     cds$tx_id <- cds_tx_id
 
     exon2cds <- .find_exon_cds(exons, cds)
@@ -1031,7 +1036,8 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
         stop_codon_start <- stop_codons$start[exon2stop_codon]
         stop_codon_end <- stop_codons$end[exon2stop_codon]
 
-        ## Exons with no CDS get the stop codon as CDS (with phase set to 0).
+        ## Exons with no CDS part get the stop codon as CDS (with phase
+        ## set to 0).
         replace_idx <- which(is.na(exon2cds) & !is.na(exon2stop_codon))
         cds_name[replace_idx] <- stop_codon_name[replace_idx]
         cds_strand[replace_idx] <- stop_codon_strand[replace_idx]
@@ -1040,8 +1046,8 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
         if (!is.null(cds$phase))
             cds_phase[replace_idx] <- 0L
 
-        ## Exons with a CDS and a stop codon have the latter merged into the
-        ## former (with phase adjusted).
+        ## Exons with a CDS part and a stop codon have the latter merged
+        ## into the former (with phase adjusted).
         merge_idx <- which(!is.na(exon2cds) & !is.na(exon2stop_codon))
         start1 <- cds_start[merge_idx]
         end1 <- cds_end[merge_idx]
