@@ -240,7 +240,7 @@ makeTxDbPackageFromUCSC <- function(
   tablename="knownGene",
   transcript_ids=NULL,    ## optional
   circ_seqs=NULL,
-  url="http://genome.ucsc.edu/cgi-bin/",
+  url="https://genome.ucsc.edu/cgi-bin/",
   goldenPath.url=getOption("UCSC.goldenPath.url"),
   taxonomyId=NA,
   miRBaseBuild=NA){
@@ -377,7 +377,7 @@ makeFDbPackageFromUCSC <- function(
     track="tRNAs",
     tablename="tRNAs",
     columns = UCSCFeatureDbTableSchema(genome, track, tablename),
-    url="http://genome.ucsc.edu/cgi-bin/",
+    url="https://genome.ucsc.edu/cgi-bin/",
     goldenPath.url=getOption("UCSC.goldenPath.url"),
     chromCol=NULL,
     chromStartCol=NULL,
